
/*
 * Copyright 2011 Google Inc.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */

#include "gl/SkNullGLContext.h"

const GrGLInterface* SkNullGLContext::createGLContext() {
    return GrGLCreateNullInterface();
};
