/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

%include tree.css

/* ::::: twisty :::::  */

treechildren::-moz-tree-indentation {
  width: 12px;
}

treechildren::-moz-tree-twisty {
  -moz-padding-end: 1px;
  width: 9px;
}

/*
  Please note that the following RTL icons are only available in Aero themes:
   * chrome://global/skin/tree/twisty-clsd-hover.png
   * chrome://global/skin/tree/twisty-open-hover.png
   * chrome://global/skin/tree/twisty-clsd-rtl.png
   * chrome://global/skin/tree/twisty-open-rtl.png
   * chrome://global/skin/tree/twisty-clsd-hover-rtl.png
   * chrome://global/skin/tree/twisty-open-hover-rtl.png
*/
treechildren::-moz-tree-twisty(hover) {
  list-style-image: url("chrome://global/skin/tree/twisty-clsd-hover.png");
}

treechildren::-moz-tree-twisty(hover, open) {
  list-style-image: url("chrome://global/skin/tree/twisty-open-hover.png");
}

treechildren:-moz-locale-dir(rtl)::-moz-tree-twisty {
  list-style-image: url("chrome://global/skin/tree/twisty-clsd-rtl.png");
}

treechildren:-moz-locale-dir(rtl)::-moz-tree-twisty(open) {
  list-style-image: url("chrome://global/skin/tree/twisty-open-rtl.png");
}

treechildren:-moz-locale-dir(rtl)::-moz-tree-twisty(hover) {
  list-style-image: url("chrome://global/skin/tree/twisty-clsd-hover-rtl.png");
}

treechildren:-moz-locale-dir(rtl)::-moz-tree-twisty(hover, open) {
  list-style-image: url("chrome://global/skin/tree/twisty-open-hover-rtl.png");
}
