# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ANDROID_PACKAGE_NAME=org.mozilla.b2g_`echo $USER`
MOZ_APP_DISPLAYNAME=B2G
MOZ_UPDATER=
